/* pG: permeable component of gas
 * bG: bath (neutral) component of gas
 * pL: permeable component of liquid
 * sL: solvent to capture pL in liquid
 * iL: intermediate species: pL+sL→iL
 * cL: captured species: iL→cL
 * bL: bath (neutral) component of liquid
 */

const int pL=0; const double pLY=0.0; 
const int bL=1; const double bLY=0.8; 
const int sL=2; const double sLY=0.2;
const int iL=3; const double iLY=0.0;
const int cL=4; const double cLY=0.0;

const int pG=0; const double pGY=0.10;
const int bG=1; const double bGY=0.90;

const int LEVEL=7;

const bool saturate=true;

double dt=1e-01; 
double steadyStateTolerance=1e-10;
double liqTimeStepFactor=1.0e-03;
double gasTimeStepFactor=1.0e-02;

double maxLength=3e+03*1.0e-04; //thousand radii
const int nVar[2]={3,2};
const int bathIndex[2]={1,1};

/*The Peclet number is a jagged array: the first index corresponds to the
  /domain and the second to the variable:*/
double Peclet[][3]={{1e+04,1e+04,1e+04},
  		    {1e+04,1e+04}};

/*The radii of the inner (lower) and outer (upper) domains:*/
double radii[2]={1.0e-04,2.0e-04};

/*The density in the inner (lower) and outer (upper) domains:*/
double density[2]={997.0,1.17};

/* The diffusion coefficient of the permeable species in the first and second
 * domains. Only their ratio really matters as the Peclet numbers above subsume
 * the diffusivities. The diffusion is typically orders of magnitude smaller in
 * liquids.*/
double alphaPerm[2]={1.6e-09,1.6e-05}; 

/* The Henry constant is defined as Y₋=HY₊ where Y₋ is the mass fraction of the
 * permeable species in first domain and the Y₊ is its mass fraction in the
 * second (upper) domain. Note that the standard definition of the
 * dimensionless Henry law constant is H'=Cl/Cg where Cl is the concentration
 * in the liquid phase and Cg is the concentration in the gas phase. This is
 * related to mass fractions as H'=ρl*Yl/ρg*Yg. So if Y₋=Yl and Y₊=Yg,
 * H=(ρg/ρl)H'.
 */
double henryConstant=0.83*(1.17/997.0);

