import pint 

u=pint.UnitRegistry()
m=u.meter
s=u.second
K=u.K
kg=u.kg
mSqPerS=u.meter*u.meter/u.second

def reynolds(v=1.0*m/s,l=1.0*m,nu=1.0*m*m/s):
    return(v*l/nu)

def prandtl(nu=1.0*m*m/s,alpha=1.0*m*m/s):
    return(nu/alpha)

def peclet(v=1.0*m/s,l=1.0*m,alpha=1.0*m*m/s):
    return(reynolds(v=v,l=l)*prandtl(alpha=alpha))

def schmidt(nu=1.0*m*m/s,D=1.0*m*m/s):
    return(prandtl(nu=nu,alpha=D))

def lewis(alpha=1.0*m*m/s,D=1.0*m*m/s):
    return(schmidt(nu=alpha,D=D))

class substance():
    def __init__(self,v=1.0*m/s,nu=1.0*m*m/s,D=1.0*m*m/s,alpha=1.0*m*m/s,l=1.0*m,rho=1.0*kg/(m*m*m),cp=1.0*m*m/(K*s*s)):
        self.v=v
        self.nu=nu
        self.D=D
        self.alpha=alpha
        self.l=l
        self.rho=rho
        self.cp=cp

    def mu(self):
        return(self.nu*self.rho)

    def lamb(self):
        return(self.alpha*self.rho*self.cp)

    def Re(self):
        return(reynolds(v=self.v,\
                        l=self.l,\
                        nu=self.nu))

    def Pr(self):
        return(prandtl(nu=self.nu,\
                       alpha=self.alpha))

    def PeT(self):
        return(peclet(v=self.v,\
                      l=self.l,\
                      alpha=self.alpha))

    def PeS(self):
        return(peclet(v=self.v,\
                      l=self.l,\
                      alpha=self.D))

    def Sc(self):
        return(schmidt(nu=self.nu,D=self.D))

    def Le(self):
        return(lewis(alpha=self.alpha,D=self.D))

    def report(self):
        print("Re  (vl/ν) =",self.Re())
        print("PeT (vl/α) =",self.PeT())
        print("PeS (vl/D) =",self.PeS())
        print("Pr  (ν/α)  =",self.Pr())
        print("Sc  (ν/D)  =",self.Sc())
        print("Le  (α/D)  =",self.Le())
        return(" ")


v=1.0e-01*m/s
l=100e-06*m

rho=997.0*kg/(m*m*m)
nu=1.0046138e-06*m*m/s
D=1.6e-09*m*m/s
cp=4186.0*m*m/(K*s*s)
lamb=0.598*kg*m/(K*s*s*s)
alpha=lamb/(rho*cp)

liq=substance(v=v,l=l,nu=nu,rho=rho,D=D,alpha=alpha,cp=cp)
print("Liq:")
liq.report()
print("----")

nu=1.547e-05*m*m/s
rho=1.178*kg/(m*m*m)
D=1.6e-05*m*m/s
cp=1.0e+03*m*m/(K*s*s)
alpha=2.0512e-05*m*m/s

gas=substance(v=v,l=l,nu=nu,rho=rho,D=D,alpha=alpha,cp=cp)

print("Gas:")
gas.report()

#--------------------------
