/* BoundaryAndInitialConditions.h */
/* Membrane Boundary Conditions */
// A iterative binary search function. It returns
// location of x in given array arr[l..r] if present,
// otherwise -1
int binarySearch(double arr[], int l, int r, double x)
{
  while (l <= r) {
    int m = l + (r - l) / 2;
    if (arr[m] == x) return m;
    if (arr[m] < x) l = m + 1;
    else r = m - 1;
  }
    return -1;
}

int findXIndex(double val){
  return(binarySearch(_membrane->xCoord,
		      0,
		      _membrane->nCells-1,
		      val));
}

double gradientMinus(double x){
  int i=findXIndex(x);
  double flux=_membrane->flux[i];
  return(flux/_membrane->tauMinus);
}

double gradientPlus(double x){
  int i=findXIndex(x);
  double flux=_membrane->flux[i];
  return(-flux/_membrane->tauPlus);
}


/* Dirichlet Boundary Conditions */

double dirichletValue(int d,
    		      int i){
  if     (d==0 && i==pL) return(pLY);
  else if(d==0 && i==bL) return(bLY);
  else if(d==0 && i==sL) return(sLY);
  else if(d==0 && i==iL) return(iLY);
  else if(d==0 && i==cL) return(cLY);
  else if(d==1 && i==pG) return(pGY);
  else if(d==1 && i==bG) return(bGY);
  else                   return(0.0);
}

/* Setting Boundary Conditions */

int _d=0;
int _i=0;

void setBoundaryConditions(scalar * Y){
  /*Loop through all species, and assign each one the appropriate boundary
   * conditions:
   *
   * First check if the species is permeable:
   *
   * If so, the first species encountered obviously belongs to the first
   * domain. So this one must be assigned its mass flux BC at the top.
   *
   * The second domain is ABOVE the first domain, so the permeable species
   * must be assigned it mass flux BC at the bottom.
   *
   * This implementation currently works only for ONE permeable species
   * across TWO separate domains. TODO:generalize this!
   */
  for(scalar s in Y){
    _d=s.domainIndex;
    _i=s.speciesIndex;
    if(s.permeable){
      if(saturate){
      	if(s.domainIndex==0){
          if(reactingWall){
            s[top]=neumann(gradientMinus(x));
          }else{
            s[top]=dirichlet(pGY*henryConstant);
          }
      	  s[bottom]=neumann(0.0);
	}else{
      	  s[top]=neumann(0.0);
      	  s[bottom]=neumann(0.0);
	}
      }else{
      	if(s.domainIndex==0){
      	  s[top]=neumann(gradientMinus(x));
      	  s[bottom]=neumann(0.0);
      	} else{
      	  s[top]=neumann(0.0);
      	  s[bottom]=neumann(gradientPlus(x));
      	}
      }
    }else{
      s[bottom]=neumann(0.0);
      s[top]=neumann(0.0);
    }
    if(s.Pe>0.0){
      s[left]=dirichlet(dirichletValue(_d,_i));
      s[right]=neumann(0.0);
    } else{
	s[left]=neumann(0.0);
        s[right]=dirichlet(dirichletValue(_d,_i));
    }
  }
}

double rPrime(double r, double gamma){
  return(r*(1.0-gamma)+gamma);
}

void setBoundaryValues(scalar * Y){

  for(scalar s in Y){
    _d=s.domainIndex;
    _i=s.speciesIndex;
    boundary({s});
  }
}


/* Setting Initial Conditions */

void initializeScalars(scalar * Y, struct membrane* m){
  for(scalar s in Y){
    _d=s.domainIndex;
    _i=s.speciesIndex;
    //if(s.domainIndex==0 && s.permeable){
    //  foreach() s[]=0.0;
    //}else{
      foreach() s[]=0.995*dirichletValue(_d,_i);
    //  foreach() s[]=0.2*dirichletValue(_d,_i);
    //}
  }
  updateMembrane(Y, m);
  setBoundaryValues(Y);
}


