/* Membrane.h */
/* Macros */
#define TLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(top){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

#define BLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(bottom){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

#define LLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(left){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

#define RLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(right){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

/* Membrane Attributes */
struct membrane{
  int nCells;     //no: of cells
  double* xCoord; //boundary x coordinate
  double* flux;   //speciesFlux
  double* YPlus;  //Y₊ species mass fraction ABOVE membrane
  double* YMinus; //Y₋ species mass fraction BELOW membrane
  double tauMinus;
  double tauPlus;
};

struct membrane* _membrane; //global scope which can be used for boundary
                            //conditions

/* Initialize Membrane */

struct membrane* initializeMembrane(){
  struct membrane *m = malloc(sizeof *m);
  m->nCells=1<<LEVEL;
  m->xCoord = malloc(m->nCells * sizeof(double));
  m->YPlus = malloc(m->nCells * sizeof(double));
  m->YMinus = malloc(m->nCells * sizeof(double));
  m->flux = malloc(m->nCells * sizeof(double));

  TLOOP(m->xCoord[i]=x)
  return(m);
}

/* Update Membrane */

void updateMembrane(scalar * Y, struct membrane* m){

  double tauMinus=0.0,tauPlus=0.0;
  double H=henryConstant;
  double dr=1.0/(double)m->nCells;
  for(scalar s in Y){
    if(s.permeable && s.domainIndex==0){
      tauMinus=s.tau;
      TLOOP(m->YMinus[i]=s[])
    } 
    else if (s.permeable && s.domainIndex==1){
      tauPlus=s.tau;
      BLOOP(m->YPlus[i]=s[])
    }
  }
  for(int i=0;i<m->nCells;i++){
    m->flux[i]=H*m->YPlus[i]-m->YMinus[i];
    m->flux[i]/=(dr/2.0)*((1.0/tauMinus)-(H/tauPlus));
  }
  m->tauMinus=tauMinus;
  m->tauPlus=tauPlus;
}

/* Update Wall */

void updateWall(scalar * Y, struct membrane* m){

  double tauMinus=0.0;
  double nu=reactionOrder;
  double k=rateCoefficient;
  for(scalar s in Y){
    if(s.permeable && s.domainIndex==0){
      tauMinus=s.tau;
      TLOOP(m->YMinus[i]=0.5*(s[]+s[1]))
    } 
  }
  for(int i=0;i<m->nCells;i++){
    m->flux[i]=-k*pow(m->YMinus[i],nu);
  }
  m->tauMinus=tauMinus;
}

/* Delete Membrane */

void freeMembrane(struct membrane* m){
  free(m->flux);
  free(m->YPlus);
  free(m->YMinus);
  free(m->xCoord);
  free(m);
}


