/* PostProcessing.h */
/* Print Solution */
void printSolution(scalar * Y){
  char buf[20];
  FILE *outputFile;
  snprintf(buf,20,"output.dat");
  outputFile=fopen(buf,"w");
  setBoundaryValues(Y);
  output_field(Y,outputFile,linear=true);
}

/* Macros */
#define TLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(top){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

#define BLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(bottom){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

#define LLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(left){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

#define RLOOP(EXPR) {int n=omp_get_max_threads();\
                     omp_set_num_threads(1); \
                     int i=0; \
                     foreach_boundary(right){\
                       EXPR;\
                       i++;}\
                     omp_set_num_threads(n); \
                    }

/* zOverL */
void zOverLArray(double zOverL[]){
  TLOOP(zOverL[i]=x)
}


/* Line Integrals */
struct Integrate_dy {
  scalar a, b;
  (const) scalar lambda;
  double tolerance;
  int nrelax, minlevel;
  scalar * res;
};

mgstats integrate_dy (struct Integrate_dy p) {
  scalar a = p.a, b = p.b;
  boundary ({b});
  scalar dbdy[];
  foreach()
    dbdy[] = (b[0,1] - b[0,-1])/(2.*Delta);
  const face vector alpha[] = {0., 1., 0.};
  const face vector d[] = {0., 0., 0.};
  boundary ((scalar *){a});
  return poissonMod (a=a, b=dbdy, alpha=alpha, lambda=p.lambda, u=d, tolerance=p.tolerance,
        	  p.nrelax, p.minlevel, p.res);
}


/* Total Mass Flux */

const double PI=3.1415926535897931e+00;

double boundaryFlux(scalar s, int leftOrRight){
  double sum=0.0;
  double adv=0.0;
  double diff=0.0;
  if(leftOrRight==0){
    LLOOP(adv=0.5*(s[]+s[ghost])*s.U*poiseuille(y,s.R1,s.R2);\
          diff=(s.alpha/maxLength)*(s[]-s[ghost])/Delta;\
          sum+=(adv-diff)*rPrime(y,s.gamma)*Delta)
  }else{
    RLOOP(adv=0.5*(s[]+s[ghost])*s.U*poiseuille(y,s.R1,s.R2);\
          diff=(s.alpha/maxLength)*(s[ghost]-s[])/Delta;\
          sum+=(adv-diff)*rPrime(y,s.gamma)*Delta)
  }
  return(sum*s.rho*2.0*PI*sq(s.R2)*(1.0-s.gamma));
}

double firstPoiseuilleMoment(double R1, double R2){
  double sum=0.0;
  LLOOP(sum+=poiseuille(y,R1,R2)*rPrime(y,R1/R2)*Delta)
  return(sum*2.0*PI*sq(R2)*(1.0-(R1/R2)));
}


/* Mixing Cup Mass Fractions */
void mixingCupMass(scalar s, double YMix[]){

  scalar a[], A[];
  double fpm=firstPoiseuilleMoment(s.R1,s.R2);
  foreach(){
    a[]=s[]*rPrime(y,s.gamma)*poiseuille(y,s.R1,s.R2)/fpm;
    a[]*=2.0*PI*sq(s.R2)*(1.0-s.gamma);
    A[]=rPrime(y,s.gamma); //guess
  }

  a[bottom]=dirichlet(0.0);
  a[top]=dirichlet(0.0);
  a[left]=neumann(0.0);
  a[right]=neumann(0.0);
  A[bottom]=dirichlet(0.0);
  boundary({a,A});

  mgstats mg=integrate_dy(A,a,tolerance=TOLERANCE);
  printStats(s, mg);

  TLOOP(YMix[i]=(A[ghost]+A[])*0.5)
}

/* Gradients at the interface */
void gradientAtInterface(scalar s, double gradY[]){
  if(s.domainIndex==0){
    TLOOP(gradY[i]=(s[ghost]-s[])/Delta)
  }else{
    BLOOP(gradY[i]=(s[]-s[ghost])/Delta)
  }
}

/* Values at the interface */
void valuesAtInterface(scalar s, double Y[]){
  if(s.domainIndex==0){
    TLOOP(Y[i]=(s[ghost]+s[])*0.5)
  }else{
    BLOOP(Y[i]=(s[ghost]+s[])*0.5)
  }
}


/* The Sherwood Number */

void calculateSherwoodNumber(scalar * Y){

  printf("Calculating Sherwood number\n");

  int nCells=_membrane->nCells;
  double tauMinus=_membrane->tauMinus;
  double tauPlus=_membrane->tauPlus;
  double zOverL[nCells];
  double Yb0[nCells];
  double Yb1[nCells];
  double Yb0Grad[nCells];
  double Yb1Grad[nCells];
  double Yb0Mix[nCells];
  double Yb1Mix[nCells];

  zOverLArray(zOverL);
  for(scalar s in Y){
    if(s.permeable && s.domainIndex==0){
      mixingCupMass(s,Yb0Mix);
      gradientAtInterface(s,Yb0Grad);
      valuesAtInterface(s,Yb0);
    }
    else if(s.permeable && s.domainIndex==1){
      mixingCupMass(s,Yb1Mix);
      gradientAtInterface(s,Yb1Grad);
      valuesAtInterface(s,Yb1);
    }
  }

  char buf[30];
  snprintf(buf,30,"sherwood.dat");
  FILE *sherwoodOutput=fopen(buf,"w");
  fprintf(sherwoodOutput,"%15s\t%15s\t%15s\t%15s\t%15s\t%15s\t%15s\n",
      			"#1","2","3", "4", "5", "6", "7");
  fprintf(sherwoodOutput,"%15s\t%15s\t%15s\t%15s\t%15s\t%15s\t%15s\t%15s\t%15s\n",
      			"#zOverL","Yb0Mix","Yb1Mix", "Yb0", "Yb1", "Sh0", "Sh1", "K0", "K1");

  for(int i=0;i<nCells;i++){
    double Sh0=Yb0Grad[i]/(Yb0[i]-Yb0Mix[i]);
    double Sh1=Yb1Grad[i]/(Yb1[i]-Yb1Mix[i]);
    double K0=tauMinus*Sh0;
    double K1=tauPlus*Sh1;
    fprintf(sherwoodOutput,"%15.6e\t%15.6e\t%15.6e\t%15.6e\t%15.6e\t%15.6e\t%15.6e\t%15.6e\t%15.6e\n",
      	      		    zOverL[i],
			    Yb0Mix[i],
			    Yb1Mix[i],
			    Yb0[i],
			    Yb1[i],
			    Sh0,Sh1,
			    K0,K1);

  }
  fclose(sherwoodOutput);
}

void printGlobals(scalar * Y){

  char buf[20];
  snprintf(buf,20,"globals.dat");
  FILE *globals=fopen(buf,"w");

  int k=0;
  for(scalar s in Y){
    fprintf(globals,"Sp: %2d "
                    "Dom: %1d "
                    "Ind: %1d "
                    "Perm: %1d "
                    "Bath: %1d "
                    "Pe: %3.2f "
                    "R1: %3.2e "
                    "R2: %3.2e "
                    "U: %3.2e "
                    "γ: %3.2f "
                    "ρ: %3.2f "
                    "α: %2.1e "
                    "θ: %4.2f "
                    "η: %3.2f "
                    "β₁: %3.2e "
                    "β₂: %3.2e "
                    "τ: %4.2e \n",
	             k,
                     s.domainIndex,
                     s.speciesIndex,
                     s.permeable,
                     s.bath,
                     s.Pe,
                     s.R1,
                     s.R2,
                     s.U,
                     s.gamma,
                     s.rho,
                     s.alpha,
                     s.theta,
                     s.eta,
                     s.beta1,
                     s.beta2,
                     s.tau);
    k++;
  }

  double fluxLeft0=0.0;
  double fluxRight0=0.0;

  double fluxLeft1=0.0;
  double fluxRight1=0.0;

  for(scalar s in Y){
    if(s.permeable && s.domainIndex==0){
      fluxLeft0=fabs(boundaryFlux(s,0));
      fluxRight0=fabs(boundaryFlux(s,1));
    }
    else if(s.permeable && s.domainIndex==1){
      fluxLeft1=fabs(boundaryFlux(s,0));
      fluxRight1=fabs(boundaryFlux(s,1));
    }
  }

  fprintf(globals, "D0 Flux Left : %15.6e\n", fluxLeft0);
  fprintf(globals, "D0 Flux Right: %15.6e\n", fluxRight0);
  fprintf(globals, "D0 Flux Diff : %15.6e\n", fabs(fluxLeft0-fluxRight0));

  fprintf(globals, "D1 Flux Left : %15.6e\n", fluxLeft1);
  fprintf(globals, "D1 Flux Right: %15.6e\n", fluxRight1);
  fprintf(globals, "D1 Flux Diff : %15.6e\n", fabs(fluxLeft1-fluxRight1));

  fclose(globals);

}



