/* Reactions.h */
/* Damkohler Numbers */

double Da1(int speciesIndex){
  int k=speciesIndex;
  double C=(alpha(0,pL)/alpha(0,k))*(Wt[k]/Wt[pL]);
  return(Da[0]*C);
}

double k1(double Da1){
  return(Da1*alpha(0,pL)*Wt[sL]/(sq(radii[0])*density[0]));
}

double Da2(int speciesIndex){
  int k=speciesIndex;
  double C=(alpha(0,iL)/alpha(0,k))*(Wt[k]/Wt[iL]);
  return(Da[1]*C);
}

double k2(double Da2){
  return(Da2*alpha(0,iL)*Wt[iL]/(sq(radii[0])));
}


/* Species Macros */
#define SLOOP(EXPR,Y) {int k=0;\
                       for(scalar s in Y){\
                         if(s.domainIndex==0){\
                           EXPR;\
                           k++;\
                         }\
                       }\
                      }

/* Rates of Progress */

void ratesOfProgress(scalar * Y, scalar omega1, scalar omega2){

  double c[nVar[0]];

  int n=omp_get_max_threads();
  omp_set_num_threads(1);

  foreach(){
    SLOOP(c[k]=s[],Y)
    omega1[]=c[pL]*c[sL];
    omega2[]=c[iL];
  }
  boundary({omega1,omega2});

  omp_set_num_threads(n);
}

void sourceTerms(scalar * Y,
                 scalar * implicit,
                 scalar * explicit){

  scalar omega1[], omega2[];
  ratesOfProgress(Y, omega1, omega2);

  scalar s,imp,exp;
  for(s,imp,exp in Y,implicit,explicit){
    if(s.domainIndex==0){
      if(s.speciesIndex==pL){
        foreach(){
          imp[]=-Da1(pL)*omega1[]/(s[]+1e-14);
          exp[]=0.0;
        }
      }
      else if(s.speciesIndex==sL){
        foreach(){
          imp[]=-Da1(sL)*omega1[]/(s[]+1e-14);
          exp[]=0.0;
        }
      }
      else if(s.speciesIndex==iL){
        foreach(){
          imp[]=-Da2(iL);
          exp[]=Da1(iL)*omega1[];
        }
      }
      else if(s.speciesIndex==cL){
        foreach(){
          imp[]=0.0;
          exp[]=Da2(cL)*omega2[];
        }
      }
      else{
        foreach(){
          imp[]=0.0;
          exp[]=0.0;
        }
      }
    }else{
      foreach(){
        imp[]=0.0;
        exp[]=0.0;
      }
    }
    boundary({imp,exp});
  }
}



