/* Scalars.h */
/* Attributes */
attribute {
  int speciesIndex;
  int domainIndex;
  bool permeable;
  bool bath;
  double alpha;
  double rho;
  double tau;
  double R1;
  double R2;
  double gamma; // R1/R2
  double Pe;
  double U;
  double theta;
  double eta;
  double beta1;
  double beta2;
}

double alpha(int domainIndex, int speciesIndex){
  int i=domainIndex;
  int k=speciesIndex;
  if(speciesIndex==0){
     return(alphaPerm[i]);
  }else{
    return(Peclet[i][0]*alphaPerm[i]/Peclet[i][k]);
  }
}

int cellCount(){
  int value=0;
  foreach(reduction(+:value))value++;
  return(value);
}

int nVarTotal(){
  int n=sizeof(nVar)/sizeof(nVar[0]);
  int count=0;
  for(int i=0;i<n;i++) count+=nVar[i];
  return(count);
}

/* Setting Attributes */

void setScalarAttributes(scalar * Y){
  int i=0,k=0;
  double Pe0=Peclet[0][0];
  double alpha0=alphaPerm[0];
  double R0=radii[0];
  double rho0=density[0];
  double L=maxLength;
  for(scalar s in Y){
    s.domainIndex=i;
    s.Pe=Peclet[i][k];
    s.rho=density[i];
    s.R2=radii[i];
    if(i==0) s.R1=0.0;
    else s.R1=radii[i-1];
    s.gamma=s.R1/s.R2;
 /* The first species in each domain is permeable
  * The α (diffusivity) of this species is the base α₀
  * All other α's must be scaled by the ratio of Peclet numbers (Pe₀/Pe_k)
  */
    if(k==0) s.permeable=true;
    else     s.permeable=false;
    s.alpha=alpha(i,k);

    s.U=s.Pe*s.alpha/R0;

    /* Define θ,η,β₁,β₂,τ:
     */
    s.theta=abs(Pe0*alpha0/s.alpha);
    s.eta=Peclet[i][k]*R0/L;
    s.beta1=sq(R0/(s.R2*(1.0-s.gamma)));
    s.beta2=sq(R0/L);
    s.tau=s.rho*s.alpha/(s.R2*(1.0-s.gamma));
    s.tau*=R0/(rho0*alpha0*Pe0);
    s.speciesIndex=k;

    /* Every domain has exactly one bath gas:
     */
    if(k==bathIndex[i]) s.bath=true;
    else 		s.bath=false;

    /* Increment species. If done with species, go to next domain.
     */
    if(k<nVar[i]-1){
      k++;
    }else{
      i++;
      k=0;
    }
  }
}



