/* TimeIntegration.h */
/* Multigrid Integration */

double newNorm=1.0;
double oldNorm=0.0;

void printStats(scalar s, mgstats m){
  printf("Domain : %2d "
         "Species: %2d "
         "resbefore: %15.6e "
         "resafter: %15.6e "
         "minlevel: %d "
         "iters: %d "
         "nrelax: %d\n", 
         s.domainIndex, s.speciesIndex, 
         m.resb, m.resa, m.minlevel,
         m.i, m.nrelax);
}

void integrate(double dt, 
    	       scalar * Y, 
	       struct membrane* m){
  face vector u[];
  face vector D[];
  scalar theta[], r[], beta[], sum0[], sum1[];
  foreach() {
    sum0[]=0.0;
    sum1[]=0.0;
  }

  scalar * implicit = NULL;
  for (size_t i = 0; i < nVarTotal(); i++) {
     scalar s = new scalar;
     implicit = list_append (implicit, s);
  }
  scalar * explicit = NULL;
  for (size_t i = 0; i < nVarTotal(); i++) {
     scalar s = new scalar;
     explicit = list_append (explicit, s);
  }
  sourceTerms(Y, implicit, explicit);

  oldNorm=newNorm;
  newNorm=0.0;
  scalar s,imp,exp;
  for(s,imp,exp in Y,implicit,explicit){
    //For BC communication:
    _d=s.domainIndex;
    _i=s.speciesIndex;
    boundary({s});
    if(!s.bath){
      //Transient term:
      foreach(){

        r[]=exp[]*rPrime(y,s.gamma);
        beta[]=imp[]*rPrime(y,s.gamma);

        theta[]=s.theta*rPrime(y,s.gamma)
                       *(s.domainIndex==0?liqTimeStepFactor:gasTimeStepFactor);
      }

      //Advection term:
      initializeVelocity (u,s.R1,s.R2);
      foreach_face(x) u.x[]*=s.eta*rPrime(y,s.gamma);

      //Diffusion term:
      foreach_face(y) D.y[]=rPrime(y,s.gamma)*s.beta1;
      foreach_face(x) D.x[]=rPrime(y,s.gamma)*s.beta2;

      mgstats mg;
      if(saturate){
        if(s.domainIndex==0){
          mg = advectionDiffusion(s,dt=dt,D=D,vel=u,
                                  theta=theta,r=r,
                                  beta=beta,
                                  tolerance=TOLERANCE, 
                                  crankNicolson=0.0);
          printStats(s, mg);
          newNorm+=mg.resb;
	}
      }else{
        mg = advectionDiffusion(s,dt=dt,D=D,vel=u,
                                theta=theta,r=r,
                                beta=beta,
                                tolerance=TOLERANCE, 
                                crankNicolson=0.0);
        printStats(s, mg);
        newNorm+=mg.resb;
      }

      if(s.domainIndex==0) {
	foreach() sum0[]+=s[];
      }else{
	foreach() sum1[]+=s[];
      }
    }
  }
  delete(implicit);
  free(implicit);
  delete(explicit);
  free(explicit);

  //Handle bath fluids:
  for(scalar s in Y){
    if(s.bath){
      if(s.domainIndex==0){
	foreach() s[]=1.0-sum0[];
      }else{
	foreach() s[]=1.0-sum1[];
      }
    }
  }
  setBoundaryValues(Y);
  
  if(reactingWall){
    updateWall(Y, m);
  }else{
    updateMembrane(Y, m);
  }
}

bool stoppingCondition(){
  if(fabs(newNorm)<steadyStateTolerance) return true;
  else return false;
}

void marchToSteadyState(scalar * Y){
  double tNow=0.0;
  do{
    integrate(dt,Y,_membrane);
    printf("t=%15.6e\n",tNow);
    tNow+=dt;
  } while(!stoppingCondition());
}



